/**
# Copyright 2024 NVIDIA CORPORATION
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
**/

package image

const (
	EnvVarCudaVersion              = "CUDA_VERSION"
	EnvVarNvidiaDisableRequire     = "NVIDIA_DISABLE_REQUIRE"
	EnvVarNvidiaDriverCapabilities = "NVIDIA_DRIVER_CAPABILITIES"
	EnvVarNvidiaImexChannels       = "NVIDIA_IMEX_CHANNELS"
	EnvVarNvidiaMigConfigDevices   = "NVIDIA_MIG_CONFIG_DEVICES"
	EnvVarNvidiaMigMonitorDevices  = "NVIDIA_MIG_MONITOR_DEVICES"
	EnvVarNvidiaRequireCuda        = NvidiaRequirePrefix + "CUDA"
	EnvVarNvidiaRequireJetpack     = NvidiaRequirePrefix + "JETPACK"
	EnvVarNvidiaVisibleDevices     = "NVIDIA_VISIBLE_DEVICES"

	NvidiaRequirePrefix = "NVIDIA_REQUIRE_"
)
