// Code generated by cmd/cgo -godefs; DO NOT EDIT.
// cgo -godefs types.go

package nvsandboxutils

type InitInput_v1 struct {
	Version uint32
	Type    uint32
	Value   [256]int8
}

type InitInput struct {
	Version uint32
	Type    uint32
	Value   [256]int8
}

type GpuFileInfo_v1 struct {
	Next        *GpuFileInfo_v1
	FileType    uint32
	FileSubType uint32
	Module      uint32
	Flags       uint32
	FilePath    *int8
}

type GpuRes_v1 struct {
	Version   uint32
	InputType uint32
	Input     [256]int8
	Files     *GpuFileInfo_v1
}

type GpuRes struct {
	Version   uint32
	InputType uint32
	Input     [256]int8
	Files     *GpuFileInfo_v1
}
