/**
# SPDX-FileCopyrightText: Copyright (c) 2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
**/

package root

import "errors"

type Versioner interface {
	Version() (string, error)
}

func FirstOf(v ...Versioner) Versioner {
	return firstVersion(v)
}

type firstVersion []Versioner

var _ Versioner = (firstVersion)(nil)

func (f firstVersion) Version() (string, error) {
	var errs error
	for _, v := range f {
		if v == nil {
			continue
		}
		version, err := v.Version()
		if err == nil {
			return version, nil
		}

		errs = errors.Join(errs, err)
	}

	return "", errs
}
