/**
# Copyright 2024 NVIDIA CORPORATION
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
**/

package lookup

import "fmt"

const (
	notFound = notFoundLocator("")
)

// A notFoundLocator always returns an ErrNotFound error.
type notFoundLocator string

func (l notFoundLocator) Locate(s string) ([]string, error) {
	return nil, l.NewError(s)
}

func (l notFoundLocator) NewError(s string) error {
	return fmt.Errorf("%s: %w", s, ErrNotFound)
}
